@@ -24,6 +24,7 @@ import os
 import re
 import shutil
 import stat
+import sys
 import time
 
 import gtk
@@ -90,6 +91,14 @@ def all_same(lst):
     return not lst or lst.count(lst[0]) == len(lst)
 
 
+def remove_blank_lines(text):
+    splits = text.splitlines()
+    lines = text.splitlines(True)
+    blanks = set([i for i, l in enumerate(splits) if not l])
+    lines = [l for i, l in enumerate(lines) if i not in blanks]
+    return ''.join(lines)
+
+
 def _files_same(files, regexes, prefs):
     """Determine whether a list of files are the same.
 
@@ -109,6 +118,8 @@ def _files_same(files, regexes, prefs):
     regexes = tuple(regexes)
     stats = tuple([StatItem._make(os.stat(f)) for f in files])
 
+    need_contents = regexes or prefs.ignore_blank_lines
+
     # If all entries are directories, they are considered to be the same
     if all([stat.S_ISDIR(s.mode) for s in stats]):
         return Same
@@ -125,11 +136,12 @@ def _files_same(files, regexes, prefs):
             return Different
 
     # If there are no text filters, unequal sizes imply a difference
-    if not regexes and not all_same([s.size for s in stats]):
+    if not need_contents and not all_same([s.size for s in stats]):
         return Different
 
     # Check the cache before doing the expensive comparison
-    cache = _cache.get((files, regexes))
+    cache_key = (files, regexes, prefs.ignore_blank_lines)
+    cache = _cache.get(cache_key)
     if cache and cache.stats == stats:
         return cache.result
 
@@ -143,9 +155,9 @@ def _files_same(files, regexes, prefs):
             data = [h.read(CHUNK_SIZE) for h in handles]
 
             # Rough test to see whether files are binary. If files are guessed
-            # to be binary, we unset regexes for speed and space reasons.
+            # to be binary, we don't examine contents for speed and space.
             if any(["\0" in d for d in data]):
-                regexes = tuple()
+                need_contents = False
 
             while True:
                 if all_same(data):
@@ -153,10 +165,10 @@ def _files_same(files, regexes, prefs):
                         break
                 else:
                     result = Different
-                    if not regexes:
+                    if not need_contents:
                         break
 
-                if regexes:
+                if need_contents:
                     for i in range(len(data)):
                         contents[i].append(data[i])
 
@@ -175,13 +187,15 @@ def _files_same(files, regexes, prefs):
     if result is None:
         result = Same
 
-    if result == Different and regexes:
+    if result == Different and need_contents:
         contents = ["".join(c) for c in contents]
         for r in regexes:
             contents = [re.sub(r, "", c) for c in contents]
+        if prefs.ignore_blank_lines:
+            contents = [remove_blank_lines(c) for c in contents]
         result = SameFiltered if all_same(contents) else Different
 
-    _cache[(files, regexes)] = CacheResult(stats, result)
+    _cache[cache_key] = CacheResult(stats, result)
     return result
 
 
@@ -272,6 +286,8 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.actiongroup.set_translation_domain("meld")
         self.actiongroup.add_actions(actions)
         self.actiongroup.add_toggle_actions(toggleactions)
+        self.main_actiongroup = None
+
         self.name_filters = []
         self.text_filters = []
         self.create_name_filters()
@@ -289,8 +305,9 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                                      "diffmap", "linkmap", "msgarea_mgr",
                                      "vbox"])
 
-        self.widget.connect("style-set", self.on_style_set)
         self.widget.ensure_style()
+        self.on_style_set(self.widget, None)
+        self.widget.connect("style-set", self.on_style_set)
 
         self.custom_labels = []
         self.set_num_panes(num_panes)
@@ -362,9 +379,13 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.update_treeview_columns(self.prefs.dirdiff_columns)
 
         for i in range(3):
-            self.treeview[i].get_selection().set_mode(gtk.SELECTION_MULTIPLE)
-            self.scrolledwindow[i].get_vadjustment().connect("value-changed", self._sync_vscroll )
-            self.scrolledwindow[i].get_hadjustment().connect("value-changed", self._sync_hscroll )
+            selection = self.treeview[i].get_selection()
+            selection.set_mode(gtk.SELECTION_MULTIPLE)
+            selection.connect('changed', self.on_treeview_selection_changed, i)
+            self.scrolledwindow[i].get_vadjustment().connect(
+                "value-changed", self._sync_vscroll)
+            self.scrolledwindow[i].get_hadjustment().connect(
+                "value-changed", self._sync_hscroll)
         self.linediffs = [[], []]
 
         self.state_filters = []
@@ -406,6 +427,8 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             self.refresh()
         elif key == "dirdiff_time_resolution_ns":
             self.refresh()
+        elif key == "ignore_blank_lines":
+            self.refresh()
 
     def update_treeview_columns(self, columns):
         """Update the visibility and order of columns"""
@@ -450,6 +473,8 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.filter_menu_button.set_label_widget(label)
 
     def on_container_switch_in_event(self, ui):
+        self.main_actiongroup = [a for a in ui.get_action_groups()
+                                 if a.get_name() == "MainActions"][0]
         melddoc.MeldDoc.on_container_switch_in_event(self, ui)
         self._create_filter_menu_button(ui)
         self.ui_manager = ui
@@ -553,10 +578,18 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
 
     def on_fileentry_activate(self, entry):
         locs = [e.get_full_path() for e in self.fileentry[:self.num_panes]]
+        locs = [l.decode('utf8') for l in locs]
         self.set_locations(locs)
 
     def set_locations(self, locations):
         self.set_num_panes(len(locations))
+        # This is difficult to trigger, and to test. Most of the time here we
+        # will actually have had UTF-8 from GTK, which has been unicode-ed by
+        # the time we get this far. This is a fallback, and may be wrong!
+        for i, l in enumerate(locations):
+            if not isinstance(l, unicode):
+                locations[i] = l.decode(sys.getfilesystemencoding())
+        # TODO: Support for blank folder comparisons should probably look here
         locations = [os.path.abspath(l or ".") for l in locations]
         self.current_path = None
         self.model.clear()
@@ -591,7 +624,10 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.scheduler.add_task(self._search_recursively_iter(path))
 
     def _search_recursively_iter(self, rootpath):
-        self.actiongroup.get_action("Hide").set_sensitive(False)
+        for t in self.treeview:
+            sel = t.get_selection()
+            sel.unselect_all()
+
         yield _("[%s] Scanning %s") % (self.label_text, "")
         prefixlen = 1 + len( self.model.value_path( self.model.get_iter(rootpath), 0 ) )
         symlinks_followed = set()
@@ -639,7 +675,8 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
 
                 for e in entries:
                     try:
-                        e = e.decode('utf8')
+                        if not isinstance(e, unicode):
+                            e = e.decode('utf8')
                     except UnicodeDecodeError:
                         approximate_name = e.decode('utf8', 'replace')
                         encoding_errors.append((pane, approximate_name))
@@ -711,8 +748,9 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         for path in sorted(expanded):
             self.treeview[0].expand_to_path(path)
         yield _("[%s] Done") % self.label_text
-        self.actiongroup.get_action("Hide").set_sensitive(True)
+
         self.scheduler.add_task(self.on_treeview_cursor_changed)
+        self.treeview[0].get_selection().select_path((0,))
 
     def _show_tree_wide_errors(self, invalid_filenames, shadowed_entries):
         header = _("Multiple errors occurred while scanning this folder")
@@ -873,6 +911,37 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             if self.current_path and self.focus_pane:
                 self.focus_pane.set_cursor(self.current_path)
 
+    def on_treeview_selection_changed(self, selection, pane):
+        if not self.treeview[pane].is_focus():
+            return
+        have_selection = bool(selection.count_selected_rows())
+        get_action = self.actiongroup.get_action
+
+        if have_selection:
+            is_valid = True
+            for path in selection.get_selected_rows()[1]:
+                state = self.model.get_state(self.model.get_iter(path), pane)
+                if state in (tree.STATE_ERROR, tree.STATE_NONEXIST):
+                    is_valid = False
+                    break
+
+            get_action("DirCompare").set_sensitive(True)
+            get_action("Hide").set_sensitive(True)
+            get_action("DirDelete").set_sensitive(is_valid)
+            get_action("DirCopyLeft").set_sensitive(is_valid and pane > 0)
+            get_action("DirCopyRight").set_sensitive(
+                is_valid and pane + 1 < self.num_panes)
+            if self.main_actiongroup:
+                act = self.main_actiongroup.get_action("OpenExternal")
+                act.set_sensitive(is_valid)
+        else:
+            for action in ("DirCompare", "DirCopyLeft", "DirCopyRight",
+                           "DirDelete", "Hide"):
+                get_action(action).set_sensitive(False)
+            if self.main_actiongroup:
+                act = self.main_actiongroup.get_action("OpenExternal")
+                act.set_sensitive(False)
+
     def on_treeview_cursor_changed(self, *args):
         pane = self._get_focused_pane()
         if pane is None:
@@ -980,7 +1049,8 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         if not rows[pane]:
             return
         if os.path.isfile(rows[pane]):
-            self.emit("create-diff", [r for r in rows if os.path.isfile(r)])
+            self.emit("create-diff", [r for r in rows if os.path.isfile(r)],
+                      {})
         elif os.path.isdir(rows[pane]):
             if view.row_expanded(path):
                 view.collapse_row(path)
@@ -1003,15 +1073,18 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
     def on_treeview_focus_in_event(self, tree, event):
         self.focus_pane = tree
         pane = self.treeview.index(tree)
-        self.actiongroup.get_action("DirCopyLeft").set_sensitive(pane > 0)
-        self.actiongroup.get_action("DirCopyRight").set_sensitive(pane+1 < self.num_panes)
-        self.actiongroup.get_action("DirDelete").set_sensitive(True)
+        self.on_treeview_selection_changed(tree.get_selection(), pane)
         tree.emit("cursor-changed")
 
     def on_treeview_focus_out_event(self, tree, event):
-        self.actiongroup.get_action("DirCopyLeft").set_sensitive(False)
-        self.actiongroup.get_action("DirCopyRight").set_sensitive(False)
-        self.actiongroup.get_action("DirDelete").set_sensitive(False)
+        for action in ("DirCompare", "DirCopyLeft", "DirCopyRight",
+                       "DirDelete", "Hide"):
+            self.actiongroup.get_action(action).set_sensitive(False)
+        try:
+            self.main_actiongroup.get_action("OpenExternal").set_sensitive(
+                False)
+        except AttributeError:
+            pass
 
     def on_button_diff_clicked(self, button):
         pane = self._get_focused_pane()
@@ -1022,7 +1095,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         for row in selected:
             row_paths = self.model.value_paths(self.model.get_iter(row))
             paths = [p for p in row_paths if os.path.exists(p)]
-            self.emit("create-diff", paths)
+            self.emit("create-diff", paths, {})
 
     def on_button_copy_left_clicked(self, button):
         self.copy_selected(-1)
