@@ -16,16 +16,17 @@
 ### Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 ### USA.
 
-import codecs
 import copy
 import functools
-import multiprocessing
+import io
 import os
 from gettext import gettext as _
-import signal
-import sys
 import time
 
+from multiprocessing import Pool
+from multiprocessing.pool import ThreadPool
+
+
 import pango
 import glib
 import gobject
@@ -50,21 +51,6 @@ from .util.compat import text_type
 from .util.sourceviewer import srcviewer
 
 
-def init_worker():
-    signal.signal(signal.SIGINT, signal.SIG_IGN)
-
-
-def matcher_worker(text1, textn):
-    matcher = matchers.InlineMyersSequenceMatcher(None, text1, textn)
-    return matcher.get_opcodes()
-
-# maxtasksperchild is new in Python 2.7; for 2.6 compat we do this
-try:
-    process_pool = multiprocessing.Pool(None, init_worker, maxtasksperchild=1)
-except TypeError:
-    process_pool = multiprocessing.Pool(None, init_worker)
-
-
 class CachedSequenceMatcher(object):
     """Simple class for caching diff results, with LRU-based eviction
 
@@ -73,7 +59,20 @@ class CachedSequenceMatcher(object):
     eviction is overly simplistic, but is okay for our usage pattern.
     """
 
+    process_pool = None
+
     def __init__(self):
+        if self.process_pool is None:
+            if os.name == "nt":
+                CachedSequenceMatcher.process_pool = ThreadPool(None)
+            else:
+                # maxtasksperchild is new in Python 2.7; this is for 2.6 compat
+                try:
+                    CachedSequenceMatcher.process_pool = Pool(
+                        None, matchers.init_worker, maxtasksperchild=1)
+                except TypeError:
+                    CachedSequenceMatcher.process_pool = Pool(
+                        None, matchers.init_worker)
         self.cache = {}
 
     def match(self, text1, textn, cb):
@@ -84,8 +83,9 @@ class CachedSequenceMatcher(object):
             def inline_cb(opcodes):
                 self.cache[(text1, textn)] = [opcodes, time.time()]
                 gobject.idle_add(lambda: cb(opcodes))
-            process_pool.apply_async(matcher_worker, (text1, textn),
-                                     callback=inline_cb)
+            self.process_pool.apply_async(matchers.matcher_worker,
+                                          (text1, textn),
+                                          callback=inline_cb)
 
     def clean(self, size_hint):
         """Clean the cache if necessary
@@ -208,6 +208,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self._scroll_lock = False
         self.linediffer = self.differ()
         self.linediffer.ignore_blanks = self.prefs.ignore_blank_lines
+        self.syncpoints = []
         self.in_nested_textview_gutter_expose = False
         self._cached_match = CachedSequenceMatcher()
         self.anim_source_id = [None for buf in self.textbuffer]
@@ -217,6 +218,13 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
 
         actions = (
             ("MakePatch", None, _("Format as patch..."), None, _("Create a patch using differences between files"), self.make_patch),
+            ("SplitAdd", None, _("Add synchronization point"), None,
+                _("Add a manual point for synchronization of changes between "
+                  "files"),
+                self.add_sync_point),
+            ("SplitClear", None, _("Clear synchronization points"), None,
+                _("Clear manual change sychronization points"),
+                self.clear_sync_points),
             ("PrevConflict", None, _("Previous conflict"), "<Ctrl>I", _("Go to the previous conflict"), lambda x: self.on_next_conflict(gtk.gdk.SCROLL_UP)),
             ("NextConflict", None, _("Next conflict"), "<Ctrl>K", _("Go to the next conflict"), lambda x: self.on_next_conflict(gtk.gdk.SCROLL_DOWN)),
             ("PushLeft",  gtk.STOCK_GO_BACK,    _("Push to left"),    "<Alt>Left", _("Push current change to the left"), lambda x: self.push_change(-1)),
@@ -246,10 +254,13 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.actiongroup.set_translation_domain("meld")
         self.actiongroup.add_actions(actions)
         self.actiongroup.add_toggle_actions(toggle_actions)
+        self.main_actiongroup = None
+
         self.findbar = findbar.FindBar(self.table)
 
-        self.widget.connect("style-set", self.on_style_set)
         self.widget.ensure_style()
+        self.on_style_set(self.widget, None)
+        self.widget.connect("style-set", self.on_style_set)
 
         self.set_num_panes(num_panes)
         gobject.idle_add( lambda *args: self.load_font()) # hack around Bug 316730
@@ -304,6 +315,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                             "conflict": lookup("conflict-outline", "#f0768b"),
                             "replace" : lookup("replace-outline", "#8bbff3")}
         self.highlight_color = lookup("highlight-bg", "#ffff00")
+        self.syncpoint_color = lookup("syncpoint-outline", "#555555")
 
         for associated in self.diffmap + self.linkmap:
             associated.set_color_scheme([self.fill_colors, self.line_colors])
@@ -314,6 +326,8 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.keymask = 0
 
     def on_container_switch_in_event(self, ui):
+        self.main_actiongroup = [a for a in ui.get_action_groups()
+                                 if a.get_name() == "MainActions"][0]
         melddoc.MeldDoc.on_container_switch_in_event(self, ui)
         # FIXME: If no focussed textview, action sensitivity will be unset
 
@@ -657,6 +671,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.focus_pane = view
         self.findbar.textview = view
         self.on_cursor_position_changed(view.get_buffer(), None, True)
+        self._set_save_action_sensitivity()
         self._set_merge_action_sensitivity()
 
     def on_textview_focus_out_event(self, view, event):
@@ -935,12 +950,21 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
     def set_merge_output_file(self, filename):
         if len(self.textbuffer) < 2:
             return
-        self.textbuffer[1].data.savefile = os.path.abspath(filename)
-        self.textbuffer[1].data.set_label(filename)
+        buf = self.textbuffer[1]
+        buf.data.savefile = os.path.abspath(filename)
+        buf.data.set_label(filename)
+        self.set_buffer_writable(buf, os.access(buf.data.savefile, os.W_OK))
         self.fileentry[1].set_filename(os.path.abspath(filename))
         self.recompute_label()
 
+    def _set_save_action_sensitivity(self):
+        pane = self._get_focused_pane()
+        modified = False if pane == -1 else self.textbuffer[pane].data.modified
+        if self.main_actiongroup:
+            self.main_actiongroup.get_action("Save").set_sensitive(modified)
+
     def recompute_label(self):
+        self._set_save_action_sensitivity()
         filenames = []
         for i in range(self.num_panes):
             filenames.append(self.textbuffer[i].data.label)
@@ -968,13 +992,17 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
            If an element is None, the text of a pane is left as is.
         """
         self._disconnect_buffer_handlers()
-        for i,f in enumerate(files):
-            if f:
-                absfile = os.path.abspath(f)
-                self.fileentry[i].set_filename(absfile)
-                self.fileentry[i].prepend_history(absfile)
-                self.textbuffer[i].reset_buffer(absfile)
-                self.msgarea_mgr[i].clear()
+        for i, f in enumerate(files):
+            if not f:
+                continue
+            if not isinstance(f, unicode):
+                files[i] = f = f.decode('utf8')
+            absfile = os.path.abspath(f)
+            self.fileentry[i].set_filename(absfile)
+            self.fileentry[i].prepend_history(absfile)
+            self.textbuffer[i].reset_buffer(absfile)
+            self.msgarea_mgr[i].clear()
+
         self.recompute_label()
         self.textview[len(files) >= 2].grab_focus()
         self._connect_buffer_handlers()
@@ -1009,7 +1037,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             buf = textbuffers[pane]
             if filename:
                 try:
-                    handle = codecs.open(filename, "rU", try_codecs[0])
+                    handle = io.open(filename, "r", encoding=try_codecs[0])
                     task = TaskEntry(filename, handle, buf, try_codecs[:],
                                      pane, False)
                     tasks.append(task)
@@ -1034,7 +1062,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                     t.codec.pop(0)
                     if len(t.codec):
                         t.buf.delete(*t.buf.get_bounds())
-                        t.file = codecs.open(t.filename, "rU", t.codec[0])
+                        t.file = io.open(t.filename, "r", encoding=t.codec[0])
                     else:
                         t.buf.delete(*t.buf.get_bounds())
                         filename = gobject.markup_escape_text(t.filename)
@@ -1060,7 +1088,10 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                             nextbit = nextbit[0:-1]
                         t.buf.insert(t.buf.get_end_iter(), nextbit)
                     else:
-                        writable = os.access(t.filename, os.W_OK)
+                        if t.buf.data.savefile:
+                            writable = os.access(t.buf.data.savefile, os.W_OK)
+                        else:
+                            writable = os.access(t.filename, os.W_OK)
                         self.set_buffer_writable(t.buf, writable)
                         t.buf.data.encoding = t.codec[0]
                         if hasattr(t.file, "newlines"):
@@ -1117,6 +1148,11 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         """Refresh the view by clearing and redoing all comparisons"""
         self._disconnect_buffer_handlers()
         self.linediffer.clear()
+
+        for buf in self.textbuffer:
+            tag = buf.get_tag_table().lookup("inline")
+            buf.remove_tag(tag, buf.get_start_iter(), buf.get_end_iter())
+
         self.queue_draw()
         self.scheduler.add_task(self._diff_files())
 
@@ -1324,6 +1360,13 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             context.paint_with_alpha(0.25)
             context.restore()
 
+        for syncpoint in [p[pane] for p in self.syncpoints]:
+            if bounds[0] <= syncpoint <= bounds[1]:
+                ypos = textview.get_y_for_line_num(syncpoint) - visible.y
+                context.rectangle(-0.5, ypos - 0.5, width + 1, 1)
+                context.set_source_color(self.syncpoint_color)
+                context.stroke()
+
         current_time = glib.get_current_time()
         new_anim_chunks = []
         for c in self.animating_chunks[pane]:
@@ -1411,11 +1454,15 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         start, end = buf.get_bounds()
         text = text_type(buf.get_text(start, end, False), 'utf8')
         if bufdata.newlines:
-            if type(bufdata.newlines) == type(""):
-                if(bufdata.newlines) != '\n':
+            if isinstance(bufdata.newlines, basestring):
+                if bufdata.newlines != '\n':
                     text = text.replace("\n", bufdata.newlines)
-            elif type(bufdata.newlines) == type(()):
-                buttons = {'\n':("UNIX (LF)",0), '\r\n':("DOS (CR-LF)", 1), '\r':("MAC (CR)",2) }
+            else:
+                buttons = {
+                    '\n': ("UNIX (LF)", 0),
+                    '\r\n': ("DOS/Windows (CR-LF)", 1),
+                    '\r': ("Mac OS (CR)", 2),
+                }
                 newline = misc.run_dialog( _("This file '%s' contains a mixture of line endings.\n\nWhich format would you like to use?") % bufdata.label,
                     self, gtk.MESSAGE_WARNING, buttonstype=gtk.BUTTONS_CANCEL,
                     extrabuttons=[ buttons[b] for b in bufdata.newlines ] )
@@ -1452,7 +1499,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         buf.data.writable = writable
         self.recompute_label()
         index = self.textbuffer.index(buf)
-        self.readonlytoggle[index].set_visible(not writable)
+        self.readonlytoggle[index].props.visible = not writable
         self.set_buffer_editable(buf, writable)
 
     def set_buffer_modified(self, buf, yesno):
@@ -1487,6 +1534,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         if self.check_save_modified() != gtk.RESPONSE_CANCEL:
             entries = self.fileentry[:self.num_panes]
             paths = [e.get_full_path() for e in entries]
+            paths = [p.decode('utf8') for p in paths]
             self.set_files(paths)
         return True
 
@@ -1743,3 +1791,31 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         anim = TextviewLineAnimation(mark0, mark1, rgba0, rgba1, 0.5)
         self.animating_chunks[src].append(anim)
 
+    def add_sync_point(self, action):
+        pane = self._get_focused_pane()
+        if pane == -1:
+            return
+
+        # Find a non-complete syncpoint, or create a new one
+        if self.syncpoints and None in self.syncpoints[-1]:
+            syncpoint = self.syncpoints.pop()
+        else:
+            syncpoint = [None] * self.num_panes
+        cursor_it = self.textbuffer[pane].get_iter_at_mark(
+            self.textbuffer[pane].get_insert())
+        syncpoint[pane] = cursor_it.get_line()
+        self.syncpoints.append(syncpoint)
+
+        valid_points = [p for p in self.syncpoints if all(p)]
+        if valid_points and self.num_panes == 2:
+            self.linediffer.syncpoints = [
+                ((p[1], p[0]), ) for p in valid_points]
+        elif valid_points and self.num_panes == 3:
+            self.linediffer.syncpoints = [
+                ((p[1], p[0]), (p[1], p[2])) for p in valid_points]
+        self.refresh_comparison()
+
+    def clear_sync_points(self, action):
+        self.syncpoints = []
+        self.linediffer.syncpoints = []
+        self.refresh_comparison()
