@@ -13,15 +13,17 @@
 
 ### You should have received a copy of the GNU General Public License
 ### along with this program; if not, write to the Free Software
-### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+### Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
+### USA.
 
 from gettext import gettext as _
 
 import gtk
 
-import filediff
-import meldbuffer
-import merge
+from . import filediff
+from . import meldbuffer
+from . import merge
+from . import recent
 
 
 class FileMerge(filediff.FileDiff):
@@ -33,6 +35,10 @@ class FileMerge(filediff.FileDiff):
         self.textview[0].set_editable(0)
         self.textview[2].set_editable(0)
 
+    def get_comparison(self):
+        comp = filediff.FileDiff.get_comparison(self)
+        return recent.TYPE_MERGE, comp[1]
+
     def _set_files_internal(self, files):
         self.textview[1].set_buffer(meldbuffer.MeldBuffer())
         for i in self._load_files(files, self.textbuffer):
@@ -47,7 +53,7 @@ class FileMerge(filediff.FileDiff):
         yield _("[%s] Merging files") % self.label_text
         merger = merge.Merger()
         step = merger.initialize(self.buffer_filtered, self.buffer_texts)
-        while step.next() is None:
+        while next(step) is None:
             yield 1
         for merged_text in merger.merge_3_files():
             yield 1
