@@ -293,6 +293,13 @@ class MeldWindow(gnomeglade.Component):
         self.actiongroup.get_action("Undo").set_sensitive(can_undo)
         self.actiongroup.get_action("Redo").set_sensitive(can_redo)
 
+        # FileDiff handles save sensitivity; it makes no sense for other modes
+        if not isinstance(newdoc, filediff.FileDiff):
+            self.actiongroup.get_action("Save").set_sensitive(False)
+            self.actiongroup.get_action("SaveAs").set_sensitive(False)
+        else:
+            self.actiongroup.get_action("SaveAs").set_sensitive(True)
+
         nbl = self.notebook.get_tab_label(newdoc.widget)
         self.widget.set_title(nbl.get_label_text() + " - Meld")
         try:
@@ -592,7 +599,8 @@ class MeldWindow(gnomeglade.Component):
         if isinstance(page, melddoc.MeldDoc):
             page.connect("label-changed", self.on_notebook_label_changed)
             page.connect("file-changed", self.on_file_changed)
-            page.connect("create-diff", lambda obj, arg: self.append_diff(arg))
+            page.connect("create-diff", lambda obj, arg, kwargs:
+                         self.append_diff(arg, **kwargs))
             page.connect("status-changed",
                          lambda obj, arg: self.statusbar.set_doc_status(arg))
 
@@ -621,21 +629,26 @@ class MeldWindow(gnomeglade.Component):
             doc.on_button_diff_clicked(None)
         return doc
 
-    def append_filediff(self, files):
+    def append_filediff(self, files,  merge_output=None):
         assert len(files) in (1, 2, 3)
         doc = filediff.FileDiff(app.prefs, len(files))
         self._append_page(doc, "text-x-generic")
         doc.set_files(files)
+        if merge_output is not None:
+            doc.set_merge_output_file(merge_output)
         return doc
 
-    def append_filemerge(self, files):
+    def append_filemerge(self, files, merge_output=None):
         assert len(files) == 3
         doc = filemerge.FileMerge(app.prefs, len(files))
         self._append_page(doc, "text-x-generic")
         doc.set_files(files)
+        if merge_output is not None:
+            doc.set_merge_output_file(merge_output)
         return doc
 
-    def append_diff(self, paths, auto_compare=False, auto_merge=False):
+    def append_diff(self, paths, auto_compare=False, auto_merge=False,
+                    merge_output=None):
         dirslist = [p for p in paths if os.path.isdir(p)]
         fileslist = [p for p in paths if os.path.isfile(p)]
         if dirslist and fileslist:
@@ -660,9 +673,9 @@ class MeldWindow(gnomeglade.Component):
         elif dirslist:
             return self.append_dirdiff(paths, auto_compare)
         elif auto_merge:
-            return self.append_filemerge(paths)
+            return self.append_filemerge(paths, merge_output=merge_output)
         else:
-            return self.append_filediff(paths)
+            return self.append_filediff(paths, merge_output=merge_output)
 
     def append_vcview(self, location, auto_compare=False):
         doc = vcview.VcView(app.prefs)
@@ -696,7 +709,8 @@ class MeldWindow(gnomeglade.Component):
         self.scheduler.add_scheduler(doc.scheduler)
         path = os.path.abspath(path)
         doc.set_location(path)
-        doc.connect("create-diff", lambda obj,arg: self.append_diff(arg))
+        doc.connect("create-diff", lambda obj, arg, kwargs:
+                    self.append_diff(arg, **kwargs))
         doc.run_diff([path])
 
     def open_paths(self, paths, auto_compare=False, auto_merge=False):
