@@ -31,7 +31,9 @@ from meld.vc._vc import \
     STATE_IGNORED, STATE_NONE, STATE_NORMAL, STATE_NOCHANGE, \
     STATE_ERROR, STATE_EMPTY, STATE_NEW, \
     STATE_MODIFIED, STATE_CONFLICT, STATE_REMOVED, \
-    STATE_MISSING, STATE_NONEXIST, STATE_MAX
+    STATE_MISSING, STATE_NONEXIST, STATE_MAX, \
+    CONFLICT_BASE, CONFLICT_LOCAL, CONFLICT_REMOTE, \
+    CONFLICT_MERGED, CONFLICT_OTHER, CONFLICT_THIS
 
 
 class DiffTreeStore(gtk.TreeStore):
@@ -102,7 +104,10 @@ class DiffTreeStore(gtk.TreeStore):
         return [self.value_path(it, i) for i in range(self.ntree)]
 
     def value_path(self, it, pane):
-        return self.get_value(it, self.column_index(COL_PATH, pane))
+        path = self.get_value(it, self.column_index(COL_PATH, pane))
+        if path is not None:
+            path = path.decode('utf8')
+        return path
 
     def column_index(self, col, pane):
         return self.ntree * col + pane
