@@ -13,7 +13,8 @@
 
 ### You should have received a copy of the GNU General Public License
 ### along with this program; if not, write to the Free Software
-### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+### Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
+### USA.
 
 import cairo
 import gobject
@@ -97,7 +98,7 @@ class EmblemCellRenderer(gtk.GenericCellRenderer):
                 # adjust the base colours accordingly
                 luma = (r + r + b + g + g + g) / 6.
                 extra_luma = (1.2 - luma) / 3.
-                r, g, b = [min(x + extra_luma, 1.) for x in r, g, b]
+                r, g, b = [min(x + extra_luma, 1.) for x in (r, g, b)]
                 context.set_source_rgba(r, g, b, 0.4)
                 context.set_operator(cairo.OPERATOR_ATOP)
                 context.paint()
