@@ -23,7 +23,7 @@
 
 import os
 import glob
-import _null
+from . import _null
 
 def load_plugins():
     _vcdir = os.path.dirname(os.path.abspath(__file__))
@@ -91,13 +91,7 @@ def get_vcs(location):
         # No plugin recognized that location, fallback to _null
         return [_null.Vc(location)]
 
-    vc_comp = lambda a, b: cmp(vc_sort_order.index(a), vc_sort_order.index(b))
-    vc_name = lambda x: x.NAME
-    vcs.sort(cmp=vc_comp, key=vc_name)
-
-    # Simplistic hack so that we don't offer both 1.7 and <1.6 SVN
-    vc_names = [plugin.NAME for plugin in vcs]
-    if "Subversion" in vc_names and "Subversion 1.7" in vc_names:
-        vcs = [plugin for plugin in vcs if plugin.NAME != "Subversion 1.7"]
+    vc_sort_key = lambda v: vc_sort_order.index(v.NAME)
+    vcs.sort(key=vc_sort_key)
 
     return vcs
