@@ -21,7 +21,8 @@
 ### (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 ### THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-import _vc
+from . import _vc
+
 
 class Vc(_vc.Vc):
 
@@ -44,11 +45,8 @@ class Vc(_vc.Vc):
     def resolved_command(self):
         return [self.CMD,"resolved"]
 
-    def lookup_files(self, dirs, files):
+    def lookup_files(self, dirs, files, directory=None):
         "files is array of (name, path). assume all files in same dir"
-        if len(files) == 0 and len(dirs) == 0:
-            return [],[]
-
-        d = map(lambda x: _vc.Dir(x[1],x[0], _vc.STATE_NONE), dirs)
-        f = map(lambda x: _vc.File(x[1],x[0], _vc.STATE_NONE, None), files)
-        return d,f
+        d = [_vc.Dir(x[1], x[0], _vc.STATE_NONE) for x in dirs]
+        f = [_vc.File(x[1], x[0], _vc.STATE_NONE) for x in files]
+        return d, f
