@@ -23,9 +23,11 @@
 ### (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 ### THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-import os
-import _vc
 import errno
+import logging
+import os
+
+from . import _vc
 
 
 class Vc(_vc.CachedVc):
@@ -79,6 +81,8 @@ class Vc(_vc.CachedVc):
         return self.root
 
     def _lookup_tree_cache(self, rootdir):
+        log = logging.getLogger(__name__)
+
         while 1:
             try:
                 entries = _vc.popen([self.CMD, "ls", "-l"],
@@ -102,19 +106,16 @@ class Vc(_vc.CachedVc):
                       os.sep + fname):
                     state = _vc.STATE_MISSING
 
-                if state == _vc.STATE_ERROR:
-                    print "WARNING: unknown state ('%s') reported by " \
-                            "'ls -l'" % mstate
             else:
                 state = _vc.STATE_ERROR
-                print "WARNING: unknown state ('%s') reported by 'ls -l' " \
-                        "(version skew?)" % mstate
+                log.warning("Unknown state '%s'", mstate)
 
             tree_state[os.path.join(self.root, fname)] = state
 
         return tree_state
 
     def _get_dirsandfiles(self, directory, dirs, files):
+        log = logging.getLogger(__name__)
 
         tree = self._get_tree_cache(directory)
 
@@ -122,7 +123,7 @@ class Vc(_vc.CachedVc):
         retdirs = []
         vcfiles = {}
 
-        for path, state in tree.iteritems():
+        for path, state in tree.items():
             mydir, name = os.path.split(path)
             if path.endswith('/'):
                 mydir, name = os.path.split(mydir)
@@ -143,12 +144,10 @@ class Vc(_vc.CachedVc):
                     if e.errno != errno.EAGAIN:
                         raise
 
-            options, tag = "", ""
             if path.endswith('/'):
                 retdirs.append(_vc.Dir(path[:-1], name, state))
             else:
-                retfiles.append(_vc.File(path, name, state, rev, tag,
-                        options))
+                retfiles.append(_vc.File(path, name, state, rev))
             vcfiles[name] = 1
 
         for f, path in files:
@@ -163,12 +162,12 @@ class Vc(_vc.CachedVc):
                                   'manifest.uuid']
 
                     if f not in ignorelist:
-                        print "WARNING: '%s' was not listed by 'ls -l'" % f
+                        log.warning("'%s' was not listed", f)
 
                 # If it ain't listed by the inventory it's not under version
                 # control
                 state = _vc.STATE_NONE
-                retfiles.append(_vc.File(path, f, state, ""))
+                retfiles.append(_vc.File(path, f, state))
 
         for d, path in dirs:
             if d not in vcfiles:
