@@ -47,6 +47,14 @@ class Vc(_vc.CachedVc):
     PATCH_INDEX_RE = "^diff --git [ac]/(.*) [bw]/.*$"
     GIT_DIFF_FILES_RE = ":(\d+) (\d+) [a-z0-9]+ [a-z0-9]+ ([ADMU])\t(.*)"
 
+    conflict_map = {
+        # These are the arguments for git-show
+        # CONFLICT_MERGED has no git-show argument unfortunately.
+        _vc.CONFLICT_BASE: 1,
+        _vc.CONFLICT_LOCAL: 2,
+        _vc.CONFLICT_REMOTE: 3,
+    }
+
     state_map = {
         "X": _vc.STATE_NONE,     # Unknown
         "A": _vc.STATE_NEW,      # New
@@ -80,6 +88,33 @@ class Vc(_vc.CachedVc):
     def revert_command(self):
         return [self.CMD,"checkout"]
 
+    def get_path_for_conflict(self, path, conflict):
+        if not path.startswith(self.root + os.path.sep):
+            raise _vc.InvalidVCPath(self, path, "Path not in repository")
+
+        if conflict == _vc.CONFLICT_MERGED:
+            # Special case: no way to get merged result from git directly
+            return path, False
+
+        path = path[len(self.root) + 1:]
+        if os.name == "nt":
+            path = path.replace("\\", "/")
+
+        args = ["git", "show", ":%s:%s" % (self.conflict_map[conflict], path)]
+        process = subprocess.Popen(args,
+                                   cwd=self.location, stdout=subprocess.PIPE,
+                                   stderr=subprocess.PIPE)
+        vc_file = process.stdout
+
+        # Error handling here involves doing nothing; in most cases, the only
+        # sane response is to return an empty temp file.
+
+        with tempfile.NamedTemporaryFile(
+                                prefix='meld-tmp-%s-' % _vc.conflicts[conflict],
+                                delete=False) as f:
+            shutil.copyfileobj(vc_file, f)
+        return f.name, True
+
     def get_path_for_repo_file(self, path, commit=None):
         if commit is None:
             commit = "HEAD"
@@ -89,6 +124,8 @@ class Vc(_vc.CachedVc):
         if not path.startswith(self.root + os.path.sep):
             raise _vc.InvalidVCPath(self, path, "Path not in repository")
         path = path[len(self.root) + 1:]
+        if os.name == "nt":
+            path = path.replace("\\", "/")
 
         obj = commit + ":" + path
         process = subprocess.Popen([self.CMD, "cat-file", "blob", obj],
