@@ -22,6 +22,7 @@ import atexit
 import tempfile
 import shutil
 import os
+import stat
 import sys
 from gettext import gettext as _
 
@@ -59,6 +60,9 @@ def cleanup_temp():
         try:
             assert os.path.exists(f) and os.path.isabs(f) and \
                    os.path.dirname(f) == temp_location
+            # Windows throws permissions errors if we remove read-only files
+            if os.name == "nt":
+                os.chmod(f, stat.S_IWRITE)
             os.remove(f)
         except:
             except_str = "{0[0]}: \"{0[1]}\"".format(sys.exc_info())
@@ -300,7 +304,15 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             # check fails don't let the user select the that version control
             # tool and display a basic error message in the drop-down menu.
             err_str = ""
-            if vc._vc.call(["which", avc.CMD]):
+
+            def vc_installed(cmd):
+                try:
+                    return not vc._vc.call(["which", cmd])
+                except OSError:
+                    if os.name == 'nt':
+                        return not vc._vc.call(["where", cmd])
+
+            if not vc_installed(avc.CMD):
                 # TRANSLATORS: this is an error message when a version control
                 # application isn't installed or can't be found
                 err_str = _("%s Not Installed" % avc.CMD)
@@ -460,15 +472,44 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                     retry_diff = True
             else:
                 for path in path_list:
-                    self.emit("create-diff", [path])
+                    self.emit("create-diff", [path], {})
 
     def run_diff(self, path_list):
         try:
             for path in path_list:
-                comp_path = self.vc.get_path_for_repo_file(path)
-                os.chmod(comp_path, 0o444)
-                _temp_files.append(comp_path)
-                self.emit("create-diff", [comp_path, path])
+                if os.path.isdir(path):
+                    self.emit("create-diff", [path], {})
+                    continue
+
+                kwargs = {}
+                status = self.vc._get_tree_cache(path).get(path, None)
+                if status == tree.STATE_CONFLICT:
+                    # We use auto merge, so we create a new temp file
+                    # for other, base and this, then set the output to
+                    # the current file.
+
+                    conflicts = (tree.CONFLICT_OTHER, tree.CONFLICT_MERGED,
+                                 tree.CONFLICT_THIS)
+                    diffs = [self.vc.get_path_for_conflict(path, conflict=c)
+                             for c in conflicts]
+                    for conflict_path, is_temp in diffs:
+                        # If this is the actual file, don't touch it.
+                        if conflict_path != path and is_temp:
+                            os.chmod(conflict_path, 0o444)
+                            _temp_files.append(conflict_path)
+                    # create-diff expects only the paths
+                    diffs = [path for path, is_temp in diffs]
+
+                    # If we want to use auto-merge or use the merged
+                    # output given by the VCS
+                    kwargs['auto_merge'] = False
+                    kwargs['merge_output'] = path
+                else:
+                    comp_path = self.vc.get_path_for_repo_file(path)
+                    os.chmod(comp_path, 0o444)
+                    _temp_files.append(comp_path)
+                    diffs = [comp_path, path]
+                self.emit("create-diff", diffs, kwargs)
         except NotImplementedError:
             for path in path_list:
                 self.scheduler.add_task(self.run_diff_iter([path]), atfront=1)
@@ -524,7 +565,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         # Remove empty entries and trailing slashes
         return [x[-1] != "/" and x or x[:-1] for x in sel if x is not None]
 
-    def _command_iter(self, command, files, refresh):
+    def _command_iter(self, command, files, refresh, working_dir=None):
         """Run 'command' on 'files'. Return a tuple of the directory the
            command was executed in and the output of the command.
         """
@@ -535,7 +576,9 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             if len(pbase) and p.startswith(pbase):
                 kill = len(pbase) + 1
             return p[kill:] or "."
-        if len(files) == 1 and os.path.isdir(files[0]):
+        if working_dir:
+            workdir = self.vc.get_working_directory(working_dir)
+        elif len(files) == 1 and os.path.isdir(files[0]):
             workdir = self.vc.get_working_directory(files[0])
         else:
             workdir = self.vc.get_working_directory( _commonprefix(files) )
@@ -556,10 +599,11 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             self.refresh_partial(workdir)
         yield workdir, r
 
-    def _command(self, command, files, refresh=1):
+    def _command(self, command, files, refresh=1, working_dir=None):
         """Run 'command' on 'files'.
         """
-        self.scheduler.add_task(self._command_iter(command, files, refresh))
+        self.scheduler.add_task(self._command_iter(command, files, refresh,
+                                                   working_dir))
 
     def _command_on_selected(self, command, refresh=1):
         files = self._get_selected_files()
@@ -573,7 +617,19 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         CommitDialog(self).run()
 
     def on_button_add_clicked(self, obj):
-        self._command_on_selected(self.vc.add_command())
+        # This is an evil hack to let CVS and SVN < 1.7 deal with the
+        # requirement of adding folders from their immediate parent.
+        if self.vc.NAME in ("CVS", "Subversion"):
+            selected = self._get_selected_files()
+            dirs = [s for s in selected if os.path.isdir(s)]
+            files = [s for s in selected if os.path.isfile(s)]
+            for path in dirs:
+                self._command(self.vc.add_command(), [path],
+                              working_dir=os.path.dirname(path))
+            if files:
+                self._command(self.vc.add_command(), files)
+        else:
+            self._command_on_selected(self.vc.add_command())
 
     def on_button_remove_clicked(self, obj):
         self._command_on_selected(self.vc.remove_command())
@@ -648,7 +704,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         if result == 0:
             for d in diffs:
                 os.chmod(d[0], 0o444)
-                self.emit("create-diff", d)
+                self.emit("create-diff", d, {})
             return True
         elif not silent:
             primary = _("Error fetching original comparison file")
@@ -812,7 +868,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
 
     def treeview_search_cb(self, model, column, key, it):
         """Callback function for searching in VcView treeview"""
-        path = model.get_value(it, tree.COL_PATH)
+        path = model.value_path(it, 0)
 
         # if query text contains slash, search in full path
         if key.find('/') >= 0:
